const svgoConfig = require('../../svgo.config.json')

import { Rule } from 'webpack'
import { assetsPath, styleRules, CSSLoaderOptions } from './utils'
import { CONFIG } from '../config'
import { $isProd } from '../env'

// Constants
const ESLINT_RC    = './node_modules/jk-standard/.eslintrc.json'
const TSLINT_RC    = './node_modules/jk-standard/tslint.json'
const EXCLUDE_REG  = /node_modules/
const SVG_SYMBOL_REGS = [
  /jk-icons\/lib\/svgs/,
  /svg-symbols/
]

export const BABEL_OPTIONS = {
  presets: [
    ['es2015', {
      loose: true,
      modules: false
    }],
    'stage-0',
    'react'
  ],
  plugins: [
    'transform-runtime',
    'transform-decorators-legacy',
    'syntax-dynamic-import',
    ['import', [{
      libraryName: 'antd-mobile'
    }]]
  ],
  cacheDirectory: true
}

const IMAGEMIN_OPTIONS = {
  plugins: [
    {
      use: 'imagemin-pngquant',
      options: {
        quality: '60-80'
      }
    }
  ]
}

export const cssLoaderOptions = (): CSSLoaderOptions => ({
  extract: $isProd,
  sourceMap: !$isProd,
  cssModules: CONFIG.CSS_MODULES
})

// Loaders
const eslintLoader: Rule = {
  enforce: 'pre',
  test: /\.(jsx?|vue)$/,
  loader: 'eslint-loader',
  exclude: EXCLUDE_REG,
  options: {
    configFile: ESLINT_RC
  }
}

const tslintLoader: Rule = {
  enforce: 'pre',
  test: /\.tsx?$/,
  loader: 'tslint-loader',
  exclude: EXCLUDE_REG,
  options: {
    configFile: TSLINT_RC
  }
}

const jsonLoader: Rule = {
  test: /\.json$/,
  loader: 'json-loader'
}

const fontLoader: Rule = {
  test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
  loader: 'url-loader',
  options: {
    limit: 10000,
    name: assetsPath('fonts/[name].[hash:7].[ext]')
  }
}

const imageLoader: Rule = {
  test: /\.(png|jpe?g|svg|gif)(\?.*)?$/,
  loader: 'file-loader',
  exclude: SVG_SYMBOL_REGS,
  options: {
    limit: 10000,
    name: assetsPath('images/[name].[hash:7].[ext]')
  }
}

const pugLoader: Rule = {
  test: /\.pug$/,
  loader: 'pug-loader'
}

const svgSymbolLoader: Rule = {
  test: /\.svg$/i,
  loader: 'svg-sprite-loader',
  include: SVG_SYMBOL_REGS,
  options: {
    symbolId: 'symbol-[name]'
  }
}

const imageminLoader: Rule = {
  test: /\.png$/,
  enforce: 'pre',
  loader: 'imagemin-loader',
  exclude: EXCLUDE_REG,
  options: IMAGEMIN_OPTIONS
}

const svgoLoader: Rule = {
  test: /\.svg$/,
  enforce: 'pre',
  loader: 'svgo-loader',
  exclude: /node_modules/,
  options: svgoConfig
}

const jsLoader: Rule = {
  test: /\.jsx?$/,
  loader: 'babel-loader',
  exclude: EXCLUDE_REG,
  options: BABEL_OPTIONS
}

const tsLoader: Rule = {
  test: /\.tsx?$/,
  loader: 'ts-loader',
  exclude: EXCLUDE_REG
}

const vueLoader: Rule = {
  test: /\.vue?$/,
  loader: 'vue-loader'
}

const cssLoaders = (): Rule[] => styleRules(cssLoaderOptions())

// Loader lists
const preLoaders: Rule[] = [
  eslintLoader,
  tslintLoader
]

const templateLoaders: Rule[] = [
  pugLoader
]

const staticLoaders: Rule[] = [
  jsonLoader,
  fontLoader,
  imageLoader,
  svgSymbolLoader
]

const staticOptimizeLoaders: Rule[] = [
  imageminLoader,
  svgoLoader
]

const compileLoaders = (): Rule[] => ([
  jsLoader,
  tsLoader,
  vueLoader,
  ...cssLoaders()
])

// @Env: Prod
if ($isProd) {
  staticLoaders.push(...staticOptimizeLoaders)
}

// @Export
export const WEBPACK_LOADERS = (): Rule[] => [
  ...preLoaders,
  ...templateLoaders,
  ...staticLoaders,
  ...compileLoaders()
]
