const ExtractTextPlugin = require('extract-text-webpack-plugin')
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const VueLoaderOptionsPlugin = require('vue-loader-options-plugin')

import * as fs from 'fs'
import * as path from 'path'
import {
  DefinePlugin,
  LoaderOptionsPlugin,
  NamedModulesPlugin,
  NoEmitOnErrorsPlugin,
  HotModuleReplacementPlugin,
  optimize
} from 'webpack'
import { BABEL_OPTIONS, cssLoaderOptions } from './loaders'
import { assetsPath, cssLoaders } from './utils'
import { CONFIG } from '../config'
import { $isProd, ENV, PROJECT_ROOT } from '../env'

// Constants
const POSTCSS_CONFIG_NAME = 'postcss.config.js'
const POSTCSS_RC = fs.existsSync(`${PROJECT_ROOT}/${POSTCSS_CONFIG_NAME}`)
  ? `${PROJECT_ROOT}/${POSTCSS_CONFIG_NAME}`
  : path.resolve(__dirname, `../../${POSTCSS_CONFIG_NAME}`)

// Loader Externel Options
const postCssOptions = () => {
  const options = [
    require('autoprefixer')({
      browsers: [
        'last 2 versions',
        'ie 9-11',
        'ios >= 8',
        'android >= 4.0'
      ]
    })
  ]

  if (CONFIG.PX_TO_REM) {
    options.push(
      require('postcss-pxtorem')({
        rootValue: 100
      })
    )
  }

  return options
}

const vueOptions = {
  loaders: cssLoaders(cssLoaderOptions)
}

// Plugins
const definePlugin = new DefinePlugin({
  'process.env': {
    NODE_ENV: JSON.stringify(ENV)
  },

  // Global variables
  __DEBUG__: !$isProd,
  __ENV__: JSON.stringify(ENV)
})

const loaderOptionsPlugin = () => new LoaderOptionsPlugin({
  options: {
    postcss: postCssOptions(),
    vue: vueOptions,
    context: __dirname
  }
})

const vueLoaderOptionsPlugin = new VueLoaderOptionsPlugin({
  babel: BABEL_OPTIONS
})

const defualtIndexHtmlPlugin = new HtmlWebpackPlugin()

const noEmirOnErrorsPlugin = new NoEmitOnErrorsPlugin()

const nameModulesPlugin = new NamedModulesPlugin()

const hotModuleReplacePlugin = new HotModuleReplacementPlugin()

const extractTextPlugin = new ExtractTextPlugin(
  assetsPath('styles/[name].[contenthash].css')
)

const compressCssPlugin = new OptimizeCssAssetsPlugin({
  cssProcessorOptions: {
    safe: true
  }
})

const compressJsPlugin = new optimize.UglifyJsPlugin({
  comments: false,
  compress: {
    dead_code: true,
    warnings: false
  }
})

const occurrenceOrderPlugin = new optimize.OccurrenceOrderPlugin(true)

// List
const devPlugins   = []
const buildPlugins = []

if (!$isProd) {
  devPlugins.push(
    noEmirOnErrorsPlugin,
    nameModulesPlugin,
    hotModuleReplacePlugin
  )
}

if ($isProd) {
  buildPlugins.push(
    extractTextPlugin,
    compressCssPlugin,
    compressJsPlugin,
    occurrenceOrderPlugin
  )
}

export const WEBPACK_PLUGINS = () => [
  definePlugin,
  loaderOptionsPlugin(),
  vueLoaderOptionsPlugin,
  defualtIndexHtmlPlugin,
  ...devPlugins,
  ...buildPlugins
]
